// This code example is created for educational purpose
// by Thorsten Thormaehlen (contact: www.thormae.de).
// It is distributed without any warranty.

package de.unimarburg.firstshaderapp;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import android.opengl.GLES20;
import android.opengl.GLSurfaceView;
import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.os.SystemClock;
import android.util.Log;

class Renderer {

  private String TAG = "de.unimarburg.firstshaderapp";
  private Context appContext;
  private enum VBOs {TriangleAll, numVBOs};
  private int[] bufID = new int[VBOs.numVBOs.ordinal()];
  private int triangleVertNo = 0;
  private int progID = 0;
  private int vertID = 0;
  private int fragID = 0;
  private int vertexLoc = 0;
  private int colorLoc = 0;
  
  private int SIZEOF_FLOAT = Float.SIZE / 8;
  
  public Renderer(Context context) {
    appContext = context;
  }
  
  public void init(GL10 unused, EGLConfig config) {
    GLES20.glEnable(GLES20.GL_DEPTH_TEST);

    setupShaders();

    // generate a Vertex Buffer Object (VBO)
    GLES20.glGenBuffers(VBOs.numVBOs.ordinal(), bufID, 0);

    float triangleVertexData[] = {
       0.0f, 0.5f, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f,
      -0.5f,-0.5f, 0.0f, 0.0f, 1.0f, 0.0f, 1.0f,
       0.5f,-0.5f, 0.0f, 0.0f, 0.0f, 1.0f, 1.0f,
    };
    triangleVertNo = 3;

    int floatItems = triangleVertNo *(3+4);
    ByteBuffer bb = ByteBuffer.allocateDirect(floatItems * SIZEOF_FLOAT);
    bb.order(ByteOrder.nativeOrder());
    FloatBuffer triangleVertexFB = bb.asFloatBuffer();
    triangleVertexFB.put(triangleVertexData);
    triangleVertexFB.flip();

    GLES20.glBindBuffer(GLES20.GL_ARRAY_BUFFER, bufID[VBOs.TriangleAll.ordinal()]);
    GLES20.glBufferData(GLES20.GL_ARRAY_BUFFER, triangleVertexFB.capacity()*SIZEOF_FLOAT, 
                    triangleVertexFB, GLES20.GL_STATIC_DRAW);
  }
  
  public void  setupShaders() {
     
    vertID = GLES20.glCreateShader(GLES20.GL_VERTEX_SHADER);
    fragID = GLES20.glCreateShader(GLES20.GL_FRAGMENT_SHADER);
     
    String vs = loadShaderSrc(R.raw.first_vert); 
    String fs = loadShaderSrc(R.raw.first_frag);
   
    GLES20.glShaderSource(vertID, vs);
    GLES20.glShaderSource(fragID, fs);

    // compile the shader
    GLES20.glCompileShader(vertID);
    GLES20.glCompileShader(fragID);

    // check for errors
    printShaderInfoLog(vertID);
    printShaderInfoLog(fragID);

    // create program and attach shaders
    progID = GLES20.glCreateProgram();
    GLES20.glAttachShader(progID, vertID);
    GLES20.glAttachShader(progID, fragID);

    // link the program
    GLES20.glLinkProgram(progID);
    // output error messages
    printProgramInfoLog(progID);

    // "inputPosition" and "inputColor" are user-provided
    // IN variables of the vertex shader.
    // Their locations are stored to be used later with
    // glEnableVertexAttribArray()
    vertexLoc = GLES20.glGetAttribLocation(progID,"inputPosition");
    colorLoc = GLES20.glGetAttribLocation(progID, "inputColor");
  }
  
  private String loadShaderSrc(int resourceId) {
    StringBuilder sb = new StringBuilder();
    try {
      InputStream is = appContext.getResources().openRawResource(resourceId);
      BufferedReader br = new BufferedReader(new InputStreamReader(is));
      String line;
      while ((line = br.readLine()) != null) {
        sb.append(line);
        sb.append('\n');
      }
      br.close();
    } catch (Exception e) {
      e.printStackTrace();
    }
    return sb.toString();
  }
 
  private void printShaderInfoLog(int obj) {
    IntBuffer infoLogLengthBuf = IntBuffer.allocate(1);
    GLES20.glGetShaderiv(obj, GLES20.GL_INFO_LOG_LENGTH, infoLogLengthBuf);
    int infoLogLength = infoLogLengthBuf.get(0);
    if (infoLogLength > 0) {
      Log.e(TAG, GLES20.glGetShaderInfoLog(obj));
    }
  }
  
  
  private void printProgramInfoLog(int obj) {
    IntBuffer infoLogLengthBuf = IntBuffer.allocate(1);
    GLES20.glGetProgramiv(obj, GLES20.GL_INFO_LOG_LENGTH, infoLogLengthBuf);
    int infoLogLength = infoLogLengthBuf.get(0);
    if (infoLogLength > 0) {
      Log.e(TAG, GLES20.glGetProgramInfoLog(obj));
    }
  }
  
  public void resize(GL10 unused, int width, int height) {
    GLES20.glViewport(0, 0, width, height);
  }

  
  public void display(GL10 unused) {

    GLES20.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
    GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT | GLES20.GL_DEPTH_BUFFER_BIT);

    GLES20.glUseProgram(progID); 
    
    // bind Triangle VBO
    GLES20.glBindBuffer(GLES20.GL_ARRAY_BUFFER, bufID[VBOs.TriangleAll.ordinal()]);
    int stride = (3+4)*SIZEOF_FLOAT;
    int offset = 0;

    // position
    GLES20.glVertexAttribPointer(vertexLoc, 3, GLES20.GL_FLOAT, false, stride, offset);
    GLES20.glEnableVertexAttribArray(vertexLoc);
    
    // color
    offset = 0 + 3*SIZEOF_FLOAT;
    GLES20.glVertexAttribPointer(colorLoc, 4, GLES20.GL_FLOAT, false, stride, offset);
    GLES20.glEnableVertexAttribArray(colorLoc);
    
    // render data
    GLES20.glDrawArrays(GLES20.GL_TRIANGLES, 0, triangleVertNo);

    GLES20.glFlush();
  }
  
 

}

class GLSurfaceViewRenderer implements GLSurfaceView.Renderer {

  private Renderer renderer;
  
  final int targetFPS = 60;
  final int targetMillis = 1000 / targetFPS;
  private long startTime = 0;
  
  public GLSurfaceViewRenderer(Context appContext) {
    renderer = new Renderer(appContext);
  }
  
  @Override
  public void onDrawFrame(GL10 gl10) {
    
    renderer.display(gl10);
    
    // this code attempts to achieve a target frames-per-second rate
    long timeTillNextRender = startTime + targetMillis;
    long sleepTime = timeTillNextRender - SystemClock.uptimeMillis();
    if (sleepTime > 0) {
      try {
        Thread.sleep(sleepTime);
      } catch (InterruptedException e) {
        e.printStackTrace();
      }
    } else {
      // rendering is too slow
    }
    startTime = SystemClock.uptimeMillis();
  }

  @Override
  public void onSurfaceChanged(GL10 gl10, int width, int height) {
    renderer.resize(gl10, width, height);
  }

  @Override
  public void onSurfaceCreated(GL10 gl10, EGLConfig config) {
    renderer.init(gl10, config);
    startTime = SystemClock.uptimeMillis();
  }

  public Renderer getRenderer() {
    return renderer;
  }
}

public class FirstShader extends Activity {

  private GLSurfaceView myGLSurfaceView;
  private GLSurfaceViewRenderer myGLSurfaceViewRenderer;

  @Override
  public void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    myGLSurfaceView = new GLSurfaceView(this);
    
    // Create an OpenGL ES 2.0 context.
    myGLSurfaceView.setEGLContextClientVersion(2);
    
    myGLSurfaceViewRenderer = new GLSurfaceViewRenderer(this);
    myGLSurfaceView.setRenderer(myGLSurfaceViewRenderer);
    // myGLSurfaceView.setRenderMode(GLSurfaceView.RENDERMODE_WHEN_DIRTY);
    setContentView(myGLSurfaceView);
  }

  @Override
  protected void onPause() {
    // Large objects in graphics memory should be de-allocated here
    // This is skipped here for simplicity of the example.
    super.onPause();
    myGLSurfaceView.onPause();
  }

  @Override
  protected void onResume() {
    super.onResume();
    // Large objects in graphics memory should be re-created here
    // if they were de-allocated on onPause().
    // This is skipped here for simplicity of the example.
    myGLSurfaceView.onResume();
  }
}
